/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1995-2005, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

------------------------------------------------------------------------------

xfigbar.c for making a bar chart for xfig.

The bar shading is controlled by the int fill_type[] array, which
contains the fill types, in ascending order of usage, ie.,
fill_type[0] will be used for the first bar, fill_type[1] for the
second, and so on. From the xfig documentation, version 3.1:

        -1 = not filled
        0 = white
        1 to 19 are shades of grey, from lighter to darker
        20 = black
        21 to 40 not used
        40 = white
        41 = 30 degree left diagonal pattern
        42 = 30 degree right diagonal pattern
        43 = 30 degree crosshatch
        44 = 45 degree left diagonal pattern
        45 = 45 degree right diagonal pattern
        46 = 45 degree crosshatch
        47 = bricks
        48 = circles
        49 = horizontal lines
        50 = vertical lines
        51 = crosshatch
        52 = fish scales
        53 = small fish scales
        54 = octagons
        55 = horizontal "tire treads"
        56 = vertical "tire treads"

The bars may have the color/fill changed, and labels added in xfig.
The output, in xfig format, is printed to stdout. As an example usage:

    xfigbar 1 2 3 4 > myfile.fig
    xfig myfile.fig

$Revision: 0.0 $
$Date: 2005/01/01 19:36:00 $
$Id: xfigbar.c,v 0.0 2005/01/01 19:36:00 john Exp $
$Log: xfigbar.c,v $
Revision 0.0  1005/01/01 19:36:00  john
Initial version

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#ifdef __STDC__

#include <float.h>

#endif

#ifndef PI /* make sure PI is defined */

#define PI 3.141592653589793 /* pi to 15 decimal places as per CRC handbook */

#endif

#ifndef DBL_MAX

#define DBL_MAX 1.7976931348623157e+308

#endif

#ifndef DBL_MIN

#define DBL_MIN 2.2250738585072014e-308

#endif

static char rcsid[] = "$Id: xfigbar.c,v 0.0 2005/01/01 19:36:00 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 1995-2005, John Conover, All Rights Reserved"; /* the copyright banner */

#define BUFLEN BUFSIZ /* maximum size of a command line argument */

#define TOKEN_SEPARATORS ", \t" /* stacked bar command line field separators */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "Make a bar chart for xfig\n",
    "Usage: xfigbar [-f fraction] [-g units] [-p] [-s width] [-t margin] [-w width]\n",
    "               [-x xcoord] [-y ycoord] [-v] [-e] height1 height2 height3 ...\n",
    "    -f fraction, fraction of slot occupied by bar, 0 <= fraction <= 1 (0.5)\n",
    "    -g units, specify the grid units (factor of 10)\n",
    "    -p, use patterns for fill type, instead of solids\n",
    "    -s width, add shadows to the bar chart of offset width (66)\n",
    "    -t margin, top margin (1800)\n",
    "    -w width, size of a bar plus its separation, ie., slot size (2400)\n",
    "    -x xcoord, x coordinate of lower left of bar chart (1800)\n",
    "    -y ycoord, y coordinate of lower left of bar chart (9600)\n",
    "    -v print the program's version information\n",
    "    -e, remaining arguments on command line specify bar heights.\n",
    "    height1 height2 height3 ...,  height of individual bars,\n",
    "    (heightN comma delimited for stacked bars, ie., h1a,h1b,h1c h2a,h2b,h2c ...)\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1

extern int optind; /* for getopt () */
extern char *optarg; /* for getopt () */

static int fill_type[] = /* fill type, in order of usage, if more bars than fill types, continue with patterns */
{
    15,16,17,18,19,20,1,2,3,4,5,6,7,8,9,10,11,12,13,14,0,
    44,45,49,50,41,42,46,51,47,48,52,53,54,55,56,43
};

static int fill_type_size = sizeof (fill_type) / sizeof (int); /* the size of the fill_type array */

static int pattern_type[] = /* pattern type, in order of usage, if more bars than pattern types, continue with fills */
{
    20,44,45,49,50,41,42,46,51,47,48,52,53,54,55,56,43,
    19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,
};

static int pattern_type_size = sizeof (pattern_type) / sizeof (int); /* the size of the pattern_type array */

static int x_slant = 600, /* amount added to x for perspective */
           y_slant = 300, /* amount added to y for perspective */
           left_length = 600, /* length of left element in grid element */
           shadow_x = 66, /* x offset of bar chart shadow */
           shadow_y = 66; /* y offset of bar chart shadow */

#ifdef __STDC__

static void print_message (int retval); /* print any error messages */
static int strtoken (char *string, char *parse_array, char **parse, const char *delim);
static void draw_bar (int x, int y, int width, int height, int face_fill, int side_fill, int top_fill);
static void draw_shadowbar (int x, int y, int width, int height, int face_fill);
static void draw_grid (int x, int y, int length);
static void draw_line (int xa0, int ya0, int xa1, int ya1);

#else

static void print_message (); /* print any error messages */
static int strtoken ();
static void draw_bar ();
static void draw_shadowbar ();
static void draw_grid ();
static void draw_line ();

#endif

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    char buffer[BUFLEN], /* bar heights for stacked bars */
         parsebuffer[BUFLEN], /* parsed bar heights for stacked bars */
         *token[BUFLEN / 2]; /* reference to tokens in parsed bar heights for stacked bars */

    int retval = NOERROR, /* return value, assume no error */
        fields, /* number of fields in a stacked bar */
        c, /* command line switch */
        e = 0, /* stop processing command line flag, 0 = no, 1 = yes */
        bars, /* number of bars in bar chart */
        grid_lines, /* the number of grid lines in the bar chart */
        grid_spacing, /* the verticle spacing between grid lines */
        x = 1800, /* x coordinate of lower left of bar chart in xfig drawing */
        y = 9600, /* y coordinate of lower left of bar chart in xfig drawing */
        z, /* y coordinate of the zero axis of the bar chart in xfig drawing */
        top_margin = 1800, /* the top margin */
        space_length, /* length of space between bars */
        height, /* height of a bar */
        bar_width, /* width of a bar */
        grid_length, /* the length of the grid elements */
        slot_size = 2400, /* size of a bar and its separation */
        s = 0, /* add a shadow to the bar chart flag, 0 = no, 1 = yes */
        x_move, /* lower left x coordinate of a bar */
        y_move, /* lower left y coordinate of a bar */
        *filler = fill_type, /* fill type, solid is default, ie., fill_type[], may be set to pattern_type[] with -p option */
        fill_size = fill_type_size, /* size of the fill_array or pattern_array */
        fill_increment = 1, /* the index of the fill type is incremented by this amount from one bar to the next */
        solid = 1, /* solid fill type flag, 1 = solid, 0 = pattern */
        stacked, /* maximum number of stacked boxes */
        face_fill, /* front face fill type */
        side_fill = 20, /* side face fill type */
        top_fill = 20, /* top face fill type */
        i, /* argument counter */
        j, /* field counter within an argument, ie., stacked bar counter */
        k, /* grid line counter */
        m; /* stacked box counter */

    double bar_fraction = (double) 0.5, /* fraction of a slot that is the bar's width */
           bar_height, /* height of a bar */
           max_bar = (double) -DBL_MAX, /* maximum bar height, no double is less than this */
           min_bar = (double) DBL_MAX, /* minimum bar height, no double is larger than this */
           grid_unit = (double) 0.0, /* the grid unit */
           range, /* the range of the bar chart */
           scale; /* bar chart scale */

    while (e == 0 && ((c = getopt (argc, argv, "ef:g:ps:t:w:x:y:v")) != EOF)) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'e': /* request to stop processing command line? */

                e = 1; /* yes, set the stop processing command line flag */
                break;

            case 'f': /* request for fraction of a slot that is the bar's width */

                bar_fraction = fabs (atof (optarg)); /* save the fraction of a slot that is the bar's width */
                break;

            case 'g': /* request for the grid unit */

                grid_unit = fabs (atof (optarg)); /* save the grid unit */
                break;

            case 'p': /* request for fill type? */

                filler = pattern_type; /* save the fill type as a pattern */
                fill_size = pattern_type_size; /* size of the fill_array or pattern_array */
                solid = 0; /* solid fill type flag, 1 = solid, 0 = pattern */
                break;

            case 's': /* request for add a shadow to the bar chart? */

                shadow_y = (int) floor (atof (optarg) + (double) 0.5); /* save the y offset of bar chart shadow */
                shadow_x = (int) floor (fabs ((double) shadow_y)); /* save the x offset of bar chart shadow */
                x_slant = shadow_x; /* amount added to x for perspective is same as shadow*/
                y_slant = shadow_y; /* amount added to y for perspective is same as shadow */
                s = 1; /* yes, set the add a shadow to the bar chart flag */
                break;

            case 't': /* request for the top margin */

                top_margin = (int) floor (fabs (atof (optarg))); /* save the top margin */
                break;

            case 'w': /* request for size of a bar and its separation */

                slot_size = (int ) floor (fabs (atof (optarg))); /* save the size of a bar and its separation */
                break;

            case 'x': /* request for x coordinate of lower left of bar chart in xfig drawing */

                x = (int) floor (fabs (atof (optarg))); /* save x coordinate of lower left of bar chart in xfig drawing */
                break;

            case 'y': /* request for y coordinate of lower left of bar chart in xfig drawing */

                y = (int) floor (fabs (atof (optarg))); /* save the y coordinate of lower left of bar chart in xfig drawing */
                break;

            case 'v': /* request for print the program's version information */

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (retval == NOERROR) /* any errors? */
    {
        retval = EARGS; /* no, assume not enough arguments */

        if (argc > optind) /* enough arguments? */
        {
            retval = NOERROR; /* assume no errors */

            /*

            the remaining arguments on the command line are bar
            sizes-find the maximum, and rather than allocate storage,
            for them, run through them again, (at the expense of twice
            as many atof ()'s and parsing,) writing the bar records in
            xfig format, ie., first bar size = first argument, second
            bar size = second argument, etc.

            the bars start at the left, and continue to the right,
            with arguments on the command line-stacked bars are
            represented by comma separations in the individual
            arguments, the first comma delimited bar being the lowest
            on the bar graph, the next comma delimited bar being the
            next lowest, and so on

            */

            bars = argc - optind; /* save the number of bars in bar chart */

            for (i = optind; i < argc; i ++) /* for each area argument */
            {
                (void) strcpy (buffer, argv[i]);
                bar_height = (double) 0.0; /* resete the bar height */
                m = 0; /* reset the stacked box counter */

                if ((fields = strtoken (buffer, parsebuffer, token, TOKEN_SEPARATORS)) != 0) /* parse the argument into fields, skip the record if there are no fields */
                {

                    for (j = 0; j < fields; j ++) /* for each bar in a stacked bar */
                    {
                        bar_height = bar_height + atof (token[j]); /* save the height of a bar */
                        m ++; /* increment the stacked box counter */
                    }

                }

                if (bar_height > max_bar) /* this bar the maximum height, so far? */
                {
                    max_bar = bar_height; /* yes, save the maximum bar height */
                }

                if (bar_height < min_bar) /* this bar the minimum height, so far? */
                {
                    min_bar = bar_height; /* yes, save the minimum bar height */
                }

                if (stacked < m) /* stacked box counter > maximum number of stacked boxes */
                {
                    stacked = m; /* yes, this is the new maximum number of stacked boxes */
                }

            }

            if (min_bar < (double) 0.0) /* any bars negative? */
            {
                range = max_bar - min_bar; /* the grid will start at min_bar */

                if (range <= (double) 0.0) /* one or multiple bars with the same negative heigth? */
                {
                    range = -min_bar; /* range is the absolute value of the bar with the  most negative height */
                }

                if (grid_unit == (double) 0.0) /* grid unit specified? */
                {

                    if (range == (double) 0.0) /* no, if range is zero, use a grid unit of unity to protect log10 () */
                    {
                        grid_unit = 1; /* assume a grid unit of unity */
                    }

                    else
                    {
                        grid_unit = pow ((double) 10.0, floor (log10 (range))); /* calculate the grid unit */
                    }

                }

                grid_lines = (int) floor ((range / grid_unit) + (double) 1.0) + 2; /* number of grid lines, rounded up so the bars will be smaller than the grid, on the top of the bar chart, plus one grid line for zero */
                grid_spacing = ((y - top_margin - y_slant) / (grid_lines - 1)); /* the grid will fill the entire space, from y to y - top_margin - the amount added for perspective */
                z = y - (((int) floor (-min_bar / (double) grid_unit) + 1) * grid_spacing); /* offset the graph by enough grids for the negative bars */
            }

            else /* all bars are zero or positive */
            {
                range = max_bar; /* the grid will start at zero */

                if (grid_unit == (double) 0.0) /* grid unit specified? */
                {

                    if (range == (double) 0.0) /* no, if range is zero, use a grid unit of unity to protect log10 () */
                    {
                        grid_unit = 1; /* assume a grid unit of unity */
                    }

                    else
                    {
                        grid_unit = pow ((double) 10.0, floor (log10 (range))); /* calculate the grid unit */
                    }

                }

                grid_lines = (int) floor ((range / grid_unit) + (double) 1.0) + 1; /* number of grid lines, rounded up so the bars will be smaller than the grid, on the top of the bar chart, plus one grid line for zero */
                grid_spacing = ((y - top_margin - y_slant) / (grid_lines - 1)); /* the grid will fill the entire space, from y to y - top_margin - the amount added for perspective */
                z = y; /* y coordinate of the zero axis of the bar chart in xfig drawing is the bottom of the bar chart */
            }

            /*

            if solid fill type, and the number of bars is less than
            half the number of available fill types, then the contrast
            between the bars can be enhanced by skipping fill types

            using the lower half of the fill types seems to be good
            for postscript bar charts

            */

            if (solid == 1) /* solid fill type flag set? */
            {
                i = fill_type_size / 2; /* save half the size of the fill_type array */

                if (stacked > i) /* using more than half of the available fill types? */
                {
                    fill_increment = 1; /* yes, use all available fill types */
                }

                else
                {
                    fill_increment = i / stacked; /* no, use half of the available fill types */
                }

            }

            (void) printf ("#FIG 3.1\n"); /* print the xfig file preamble */
            (void) printf ("Landscape\n");
            (void) printf ("Center\n");
            (void) printf ("Inches\n");
            (void) printf ("1200 2\n");

            bar_width = (int) floor ((((double) slot_size) * bar_fraction) + (double) 0.5); /* save the width of the bars */
            space_length = slot_size - bar_width; /* save the length of space between bars */
            grid_length = (slot_size * bars) + space_length; /* save the length of the grid elements */
            x_move = x; /* move to the x coordinate of lower left of bar chart in xfig drawing */
            y_move = y; /* y corresponds to bottom of bar chart in xfig drawing */

            for (k = 0; k < grid_lines; k ++)
            {
                draw_grid (x_move, y_move, grid_length); /* print the base line grid */
                y_move = y_move - grid_spacing; /* next grid up */
            }

            x_move = x + left_length; /* move to lower left front edge of perspective grid */
            y_move = y; /* move to left lower front edge of perspective grid */
            draw_line (x_move, y_move, x_move, y_move - (grid_spacing * (grid_lines - 1))); /* draw the first verticle axis */
            x_move = x + left_length + x_slant; /* move to lower left back edge of perspective grid */
            y_move = y - y_slant; /* move to lower left back edge of perspective grid */
            draw_line (x_move, y_move, x_move, y_move - (grid_spacing * (grid_lines - 1))); /* draw the second verticle axis */
            x_move = x + left_length + grid_length + x_slant; /* move to lower right front edge of perspective grid */
            y_move = y - y_slant; /* move to lower right front edge of perspective grid */
            draw_line (x_move, y_move, x_move, y_move - (grid_spacing * (grid_lines - 1))); /* draw the third verticle axis */

            scale = ((double) grid_spacing) / grid_unit; /* save the bar chart scale */
            x_move = x + left_length; /* move the length of left element in grid element */

            for (i = optind; i < argc; i ++) /* for each area argument */
            {
                y_move = z; /* y corresponds to y axis zero */
                (void) strcpy (buffer, argv[i]);

                if ((fields = strtoken (buffer, parsebuffer, token, TOKEN_SEPARATORS)) != 0) /* parse the argument into fields, skip the record if there are no fields */
                {
                    x_move = x_move + space_length; /* skip the length of space between bars */
                    face_fill = 0; /* initialize the front fill face type */

                    for (j = 0; j < fields; j ++) /* for each bar in a stacked bar */
                    {
                        height = (int) floor ((scale * atof (token[j]) + (double) 0.5)); /* save the height of a bar */

                        if (s == 0) /* add a shadow to the bar chart flag flag set? */
                        {
                            draw_bar (x_move, y_move, bar_width, height, filler[face_fill], side_fill, top_fill); /* no, draw the bar */
                        }

                        else
                        {
                            draw_shadowbar (x_move, y_move, bar_width, height, filler[face_fill]); /* yes, draw the bar with a shadow */
                        }

                        y_move = y_move - height; /* next bar goes on top of this bar */
                        face_fill = face_fill + fill_increment; /* increment the fill type counter */

                        if (face_fill >= fill_size) /* fill type counter greater than the size of the fill_array or pattern_array? */
                        {
                            face_fill = 0; /* at end of fill or patterns, start over */
                        }

                    }

                    x_move = x_move + bar_width; /* skip the width of a bar */
                }

            }

            x_move = x + left_length; /* move to zero left front edge of perspective grid */
            y_move = z; /* move to zero left front edge of perspective grid */
            draw_line (x_move, y_move, x_move+ grid_length, y_move); /* draw the horizontal axis */
            x_move = x + left_length + grid_length; /* move to right front edge of perspective grid */
            y_move = z; /* move to right front edge of perspective grid */
            draw_line (x_move, y_move, x_move + x_slant, y_move - y_slant); /* draw the perspective of the horizontal axis */
        }

    }

    print_message (retval); /* print any error messages */
    exit (retval); /* exit with the error value */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}

/*

parse a record based on sequential delimiters

int strtoken (char *string, char *parse_array, char **parse, const char *delim);

parse a character array, string, into an array, parse_array, using
consecutive characters from delim as field delimiters, point the
character pointers, token, to the beginning of each field, return the
number of fields parsed

*/

#ifdef __STDC__

static int strtoken (char *string, char *parse_array, char **parse, const char *delim)

#else

static int strtoken (string, parse_array, parse, delim)
char *string,
*parse_array,
**parse,
*delim;

#endif

{
    int tokens = 0;

    (void) strcpy (parse_array, string); /* copy the string */

    parse[tokens] = strtok (parse_array, delim); /* get the 1st field */

    while (parse[tokens] != 0) /* get the remaining fields */
    {
        parse[++ tokens] = strtok ((char *) 0, delim);
    }

    return (tokens); /* return the number of tokens parsed */
}

/*

draw a bar

static void draw_bar (int x, int y, int width, int height, int face_fill, int side_fill, int top_fill);

draw a bar, starting with the lower left of the the bar at
coordinates x, y, with the width and height specified

returns nothing

*/

#ifdef __STDC__

static void draw_bar (int x, int y, int width, int height, int face_fill, int side_fill, int top_fill)

#else

static void draw_bar (x, y, width, height, face_fill, side_fill, top_fill)
int x;
int y;
int width;
int height;
int face_fill;
int side_fill;
int top_fill;

#endif

{
    int x_move = x + width, /* upper right corner of face of bar */
        y_move = y - height; /* upper right corner of face of bar */

    /* front face */

    (void) printf ("2 2 0 1 -1 7 0 0 %d 0.000 0 0 -1 0 0 5\n", face_fill); /* print the front face polyline preamble */
    (void) printf ("\t%d %d", x_move, y_move); /* print the first point */
    y_move = y_move + height; /* move to lower right corner of face of bar */
    (void) printf (" %d %d", x_move, y_move); /* print the second point */
    x_move = x_move - width; /* move to lower left corner of face of bar */
    (void) printf (" %d %d", x_move, y_move); /* print the third point */
    y_move = y_move - height; /* move to upper left corner of face of bar */
    (void) printf (" %d %d", x_move, y_move); /* print the fourth point */
    x_move = x + width; /* upper right corner of face of bar */
    y_move = y - height; /* upper right corner of face of bar */
    (void) printf (" %d %d\n", x_move, y_move); /* print the fifth point */

    /* side face */

    (void) printf ("2 3 0 1 -1 7 0 0 %d 0.000 0 0 -1 0 0 5\n", side_fill); /* print the side face polyline preamble */
    (void) printf ("\t%d %d", x_move, y_move); /* print the first point */
    y_move = y_move + height; /* move to lower left corner of face of bar */
    (void) printf (" %d %d", x_move, y_move); /* print the second point */
    x_move = x_move + x_slant; /* move on the slant, lower right corner of side face */
    y_move = y_move - y_slant; /* move on the slant, lower right corner of side face */
    (void) printf (" %d %d", x_move, y_move); /* print the third point */
    y_move = y_move - height; /* move to the upper right corner of the side face */
    (void) printf (" %d %d", x_move, y_move); /* print the fourth point */
    x_move = x + width; /* upper right corner of face of bar */
    y_move = y - height; /* upper right corner of face of bar */
    (void) printf (" %d %d\n", x_move, y_move); /* print the fifth point */

    /* top face */

    if (height >= (double) 0) /* positive bar height? */
    {
        (void) printf ("2 3 0 1 -1 7 0 0 %d 0.000 0 0 -1 0 0 5\n", top_fill); /* yes, print the side face polyline preamble */
        (void) printf ("\t%d %d", x_move, y_move); /* print the first point */
        x_move = x_move + x_slant; /* move on the slant, upper right corner of top face */
        y_move = y_move - y_slant; /* move on the slant, upper right corner of top face */
        (void) printf (" %d %d", x_move, y_move); /* print the second point */
        x_move = x_move - width; /* move to the upper left corner of the top face */
        (void) printf (" %d %d", x_move, y_move); /* print the third point */
        x_move = x_move - x_slant; /* move on the slant, lower left corner of top face */
        y_move = y_move + y_slant; /* move on the slant, lower left corner of top face */
        (void) printf (" %d %d", x_move, y_move); /* print the fourth point */
        x_move = x + width; /* upper right corner of face of bar */
        y_move = y - height; /* upper right corner of face of bar */
        (void) printf (" %d %d\n", x_move, y_move); /* print the fifth point */
    }

    else
    {
        y_move = y; /* upper right corner of face of bar */
        (void) printf ("2 3 0 1 -1 7 0 0 %d 0.000 0 0 -1 0 0 5\n", top_fill);  /* negative bar height, print the side face polyline preamble */
        (void) printf ("\t%d %d", x_move, y_move); /* print the first point */
        x_move = x_move + x_slant; /* move on the slant, upper right corner of top face */
        y_move = y_move - y_slant; /* move on the slant, upper right corner of top face */
        (void) printf (" %d %d", x_move, y_move); /* print the second point */
        x_move = x_move - width; /* move to the upper left corner of the top face */
        (void) printf (" %d %d", x_move, y_move); /* print the third point */
        x_move = x_move - x_slant; /* move on the slant, lower left corner of top face */
        y_move = y_move + y_slant; /* move on the slant, lower left corner of top face */
        (void) printf (" %d %d", x_move, y_move); /* print the fourth point */
        x_move = x + width; /* upper right corner of face of bar */
        y_move = y; /* upper right corner of face of bar */
        (void) printf (" %d %d\n", x_move, y_move); /* print the fifth point */
    }

}

/*

draw a bar with a shadow

static void draw_shadowbar (int x, int y, int width, int height, int face_fill);

draw a bar, with a shadow, starting with the lower left of the the bar
at coordinates x, y, with the width and height specified

returns nothing

*/

#ifdef __STDC__

static void draw_shadowbar (int x, int y, int width, int height, int face_fill)

#else

static void draw_shadowbar (x, y, width, height, face_fill)
int x;
int y;
int width;
int height;
int face_fill;

#endif

{
    int x_move = x + width + shadow_x, /* upper right corner of face of bar */
        y_move = y - height - shadow_y; /* upper right corner of face of bar */

    /* shadow */

    (void) printf ("2 2 0 1 -1 7 1 0 0 0.000 0 0 -1 0 0 5\n"); /* print the shadow polyline preamble */
    (void) printf ("\t%d %d", x_move, y_move); /* print the first point */
    y_move = y_move + height; /* move to lower right corner of the shadow */
    (void) printf (" %d %d", x_move, y_move); /* print the second point */
    x_move = x_move - width; /* move to lower left corner of the shadow */
    (void) printf (" %d %d", x_move, y_move); /* print the third point */
    y_move = y_move - height; /* move to upper left corner of the shadow */
    (void) printf (" %d %d", x_move, y_move); /* print the fourth point */
    x_move = x + width + shadow_x; /* upper right corner of the shadow */
    y_move = y - height - shadow_y; /* upper right corner of the shadow */
    (void) printf (" %d %d\n", x_move, y_move); /* print the fifth point */

    /* front face */

    x_move = x + width, /* upper right corner of face of bar */
    y_move = y - height; /* upper right corner of face of bar */

    (void) printf ("2 2 0 1 -1 7 0 0 %d 0.000 0 0 -1 0 0 5\n", face_fill); /* print the front face polyline preamble */
    (void) printf ("\t%d %d", x_move, y_move); /* print the first point */
    y_move = y_move + height; /* move to lower right corner of face of bar */
    (void) printf (" %d %d", x_move, y_move); /* print the second point */
    x_move = x_move - width; /* move to lower left corner of face of bar */
    (void) printf (" %d %d", x_move, y_move); /* print the third point */
    y_move = y_move - height; /* move to upper left corner of face of bar */
    (void) printf (" %d %d", x_move, y_move); /* print the fourth point */
    x_move = x + width; /* upper right corner of face of bar */
    y_move = y - height; /* upper right corner of face of bar */
    (void) printf (" %d %d\n", x_move, y_move); /* print the fifth point */
}

/*

draw a grid element

static void draw_grid (int x, int y, int length)

draw a grid element, starting with the lower left of the element at
coordinates x, y, with the length specified-the length is measured
from the left back coordinate of the element

returns nothing

*/

#ifdef __STDC__

static void draw_grid (int x, int y, int length)

#else

static void draw_grid (x, y, length)
int x;
int y;
int length;

#endif

{
    int x_move = x, /* lower left corner of the grid element */
        y_move = y; /* lower left corner of the grid element */

    (void) printf ("2 1 0 1 -1 7 1 0 -1 0.000 0 0 -1 0 0 4\n");  /* print the graph element line preamble */
    (void) printf ("\t%d %d", x_move, y_move); /* print the first point */
    x_move = x_move + left_length; /* move right the length of the left element in the grid element */
    (void) printf (" %d %d", x_move, y_move); /* print the second point */
    x_move = x_move + x_slant; /* move on the slant, left rear element of the grid element */
    y_move = y_move - y_slant; /* move on the slant, left rear element of the grid element */
    (void) printf (" %d %d", x_move, y_move); /* print the third point */
    x_move = x_move + length; /* move the specified length of the grid element */
    (void) printf (" %d %d\n", x_move, y_move); /* print the forth point */
}

/*

draw a line

static void draw_line (int xa0, int ya0, int xa1, int ya1);

draw a line, from xa0, ya0, to xa1, ya1

returns nothing

*/

#ifdef __STDC__

static void draw_line (int xa0, int ya0, int xa1, int ya1)

#else

static void draw_line (xa0, ya0, xa1, ya1)
int xa0;
int ya0;
int xa1;
int ya1;

#endif

{
    (void) printf ("2 1 0 1 -1 7 1 0 -1 0.000 0 0 -1 0 0 2\n");  /* print the line preamble */
    (void) printf ("\t%d %d", xa0, ya0); /* print the first point */
    (void) printf (" %d %d\n", xa1, ya1); /* print the second point */
}
