/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1995-2005, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

------------------------------------------------------------------------------

xfigpie.c for making a pie chart for xfig.

The slice shading is controlled by the int fill_type[] array, which
contains the fill types, in ascending order of usage, ie.,
fill_type[0] will be used for the first slice, fill_type[1] for the
second, and so on. From the xfig documentation, version 3.1:

        -1 = not filled
        0 = white
        1 to 19 are shades of grey, from lighter to darker
        20 = black
        21 to 40 not used
        40 = white
        41 = 30 degree left diagonal pattern
        42 = 30 degree right diagonal pattern
        43 = 30 degree crosshatch
        44 = 45 degree left diagonal pattern
        45 = 45 degree right diagonal pattern
        46 = 45 degree crosshatch
        47 = bricks
        48 = circles
        49 = horizontal lines
        50 = vertical lines
        51 = crosshatch
        52 = fish scales
        53 = small fish scales
        54 = octagons
        55 = horizontal "tire treads"
        56 = vertical "tire treads"

The slices may be detached, color/fill changed, and labels added in
xfig.  The output, in xfig format, is printed to stdout. As an example
usage:

    xfigpie 1 2 3 4 > myfile.fig
    xfig myfile.fig

$Revision: 0.0 $
$Date: 2005/01/18 19:36:00 $
$Id: xfigpie.c,v 0.0 2005/01/18 19:36:00 john Exp $
$Log: xfigpie.c,v $
Revision 0.0  2005/01/18 19:36:00  john
Initial version

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#ifdef __STDC__

#include <float.h>

#endif

#ifndef PI /* make sure PI is defined */

#define PI 3.141592653589793 /* pi to 15 decimal places as per CRC handbook */

#endif

static char rcsid[] = "$Id: xfigpie.c,v 0.0 2005/01/18 19:36:00 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 1995-2005, John Conover, All Rights Reserved"; /* the copyright banner */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "Make a pie chart for xfig\n",
    "Usage: xfigpie [-p] [-r radius] [-s width] [-x xcenter] [-y ycenter]\n",
    "               [-v] area1 area2 area3 ...\n",
    "    -p, use patterns for fill type, instead of solids\n",
    "    -r radius, radius of pie chart (2400)\n",
    "    -s width, add a shadow to the pie chart of offset width (66)\n",
    "    -x xcenter, x offset of center of pie chart (4800)\n",
    "    -y ycenter, y offset of center of pie chart (4800)\n",
    "    -v print the program's version information\n",
    "    area1 area2 area3 ...,  area of individual slices\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1

extern int optind; /* for getopt () */
extern char *optarg; /* for getopt () */

static int fill_type[] = /* fill type, in order of usage, if more slices than fill types, continue with patterns */
{
    20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,
    44,45,49,50,41,42,46,51,47,48,52,53,54,55,56,43
};

static int fill_type_size = sizeof (fill_type) / sizeof (int); /* the size of the fill_type array */

static int pattern_type[] = /* pattern type, in order of usage, if more slices than pattern types, continue with fills */
{
    20,44,45,49,50,41,42,46,51,47,48,52,53,54,55,56,43,
    19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,
};

static int pattern_type_size = sizeof (pattern_type) / sizeof (int); /* the size of the pattern_type array */

#ifdef __STDC__

static void print_message (int retval); /* print any error messages */

#else

static void print_message (); /* print any error messages */

#endif

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    int retval = NOERROR, /* return value, assume no error */
        c, /* command line switch */
        i, /* counter */
        s = 0, /* add a shadow to the pie chart flag, 0 = no, 1 = yes */
        fill = 0, /* fill type counter */
        last, /* last slice of pie */
        *filler = fill_type, /* fill type, solid is default, ie., fill_type[], may be set to pattern_type[] with -p option */
        solid = 1, /* solid fill type flag, 1 = solid, 0 = pattern */
        slices, /* number of slices in pie chart */
        fill_increment = 1, /* the index of the fill type is incremented by this amount from one slice to the next */
        fill_size = fill_type_size, /* size of the fill_array or pattern_array */
        center_x = 4800, /* x center of pie chart in xfig drawing */
        center_y = 4800, /* y center of pie chart in xfig drawing */
        radius = 2400, /* radius of pie chart in xfig drawing */
        shadow_x = 66, /* x offset of pie chart shadow */
        shadow_y = 66; /* y offset of pie chart shadow */

    double sum = (double) 0.0, /* sum of areas of pie chart */
           sumangle = (double) 0.0, /* sum of the angles in the pie chart */
           halfangle; /* half of the angle of a slice */

    while ((c = getopt (argc, argv, "pr:s:x:y:v")) != EOF) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'p': /* request for fill type? */

                filler = pattern_type; /* save the fill type as a pattern */
                fill_size = pattern_type_size; /* size of the fill_array or pattern_array */
                solid = 0; /* solid fill type flag, 1 = solid, 0 = pattern */
                break;

            case 'r': /* request for radius of pie chart in xfig drawing */

                radius = (int) floor (fabs (atof (optarg)) + (double) 0.5); /* save the radius of pie chart in xfig drawing */
                break;

            case 's': /* request for add a shadow to the pie chart? */

                shadow_y = (int) floor (atof (optarg) + (double) 0.5); /* save the y offset of the pie chart shadow */
                shadow_x = (int) floor (fabs ((double) shadow_y)); /* save the x offset of the pie chart shadow */
                s = 1; /* yes, set the add a shadow to the pie chart flag */
                break;

            case 'x': /* request for x center of pie chart in xfig drawing */

                center_x = (int) floor (fabs (atof (optarg)) + (double) 0.5); /* save the x center of pie chart in xfig drawing */
                break;

            case 'y': /* request for y center of pie chart in xfig drawing */

                center_y = (int) floor (fabs (atof (optarg)) + (double) 0.5); /* save the y center of pie chart in xfig drawing */
                break;

            case 'v': /* request for print the program's version information */

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (radius > center_x) /* pie chart fit in the xfig drawing */
    {
        retval = EARGS; /* no, assume not enough arguments */
    }

    if (radius > center_y) /* pie chart fit in the xfig drawing */
    {
        retval = EARGS; /* no, assume not enough arguments */
    }

    if (radius < 10) /* radius less than 10 to protect xfig? */
    {
        retval = EARGS; /* no, assume not enough arguments */
    }

    if (retval == NOERROR) /* any errors? */
    {
        retval = EARGS; /* no, assume not enough arguments */

        if (argc > optind) /* enough arguments? */
        {
            retval = NOERROR; /* assume no errors */

            /*

            the remaining arguments on the command line are slice
            sizes-sum them, and rather than allocate storage for them,
            run through them again, (at the expense of twice as many
            atof ()'s,) writing the slice records in xfig format, ie.,
            first slice size = first argument / sum, second slice size
            = second argument / sum, etc.

            the slices start at 12 noon, and continue clockwise, with
            arguments on the command line, having a fill that is in
            the same order as fill_type[], or patter_type[]

            the first x coordinate of a slice is calculated as center
            + (sin (sumangle) * radius) and center - (cos (sumangle) *
            radius), for the x and y coordinates, respectively, then
            half the slice's radial measure is added to sumangle, and
            the second set of coordinates are calculated, and then the
            remaining half of the slice's radial measure is added to
            sumangle, and the third set of coordinates are calculated

            the process is repeated, clockwise, for each slice

            the last slice size is adjusted to take up the "slop" in
            the roundoff/numerical precision errors of the sizes of
            the other slices-since xfig calculates a virtual center
            for the arc of a pie slice, this will keep the pie edges
            straight

            if more fill types are required than are available, roll
            over the fill type array to the first fill type

            */

            slices = argc - optind; /* save the number of slices in pie chart */

            for (i = optind; i < argc; i ++) /* for each area argument */
            {
                sum = sum + atof (argv[i]); /* find the sum of the areas */
            }

            (void) printf ("#FIG 3.1\n"); /* print the xfig file preamble */
            (void) printf ("Landscape\n");
            (void) printf ("Center\n");
            (void) printf ("Inches\n");
            (void) printf ("1200 2\n");
            last = argc - 1; /* save the last slice of pie */

            /*

            if solid fill type, and the number of slices is less than
            half the number of available fill types, then the contrast
            between the slices can be enhanced by skipping fill types

            using the lower half of the fill types seems to be good
            for postscript pie charts

            */

            if (solid == 1) /* solid fill type flag set? */
            {
                i = fill_type_size / 2; /* save half the size of the fill_type array */

                if (slices > i) /* using more than half of the available fill types? */
                {
                    fill_increment = 1; /* yes, use all available fill types */
                }

                else
                {
                    fill_increment = i / slices; /* no, use half of the available fill types */
                }

            }

            if (s == 1) /* add a shadow to the pie chart flag set? */
            {
                (void) printf ("1 3 0 1 -1 7 1 0 0 0.000 1 0.0000"); /* yes, print the circle preamble */
                (void) printf (" %d %d %d %d %d %d %d %d\n", center_x + shadow_x, center_y - shadow_y, radius, radius, \
                    center_x + shadow_x, center_y - shadow_y, \
                    center_x + shadow_x + (int) floor ((sin (PI / (double) 4.0) * (double) radius) + (double) 0.5), \
                    center_y - shadow_y + (int) floor ((cos (PI / (double) 4.0) * (double) radius) + (double) 0.5));
            }

            for (i = optind; i < argc; i ++) /* for each area argument */
            {
                halfangle = (atof (argv[i]) / sum) * PI; /* half the angle of the slice, in radians */
                (void) printf ("5 2 0 1 -1 7 0 0 %d 0.000 0 0 0 0 %d.000 %d.000", filler[fill], center_x, center_y); /* print the slice preamble */
                (void) printf (" %d %d", center_x + (int) floor ((sin (sumangle) * (double) radius) + (double) 0.5), center_y - (int) floor ((cos (sumangle) * (double) radius) + (double) 0.5)); /* print the coordinates of the first position */
                sumangle = sumangle + halfangle; /* increment the sum of the angles in the pie chart by half the angle of the slice, in radians */
                (void) printf (" %d %d", center_x + (int) floor ((sin (sumangle) * (double) radius) + (double) 0.5), center_y - (int) floor ((cos (sumangle) * (double) radius) + (double) 0.5)); /* print the coordinates of the second position */
                sumangle = sumangle + halfangle; /* increment the sum of the angles in the pie chart by half the angle of the slice, in radians */

                if (i < last) /* this slice the last slice? */
                {
                    (void) printf (" %d %d\n", center_x + (int) floor ((sin (sumangle) * (double) radius) + (double) 0.5), center_y - (int) floor ((cos (sumangle) * (double) radius) + (double) 0.5)); /* no, print the coordinates of the third position */
                }

                else
                {
                    (void) printf (" %d %d\n", center_x + (int) floor ((sin ((double) 0.0) * radius) + (double) 0.5), center_y - (int) floor ((cos ((double) 0.0) * radius) + (double) 0.5)); /* yes, print the coordinates of the third position, which are identical to the first position of the first slice */
                }

                fill = fill + fill_increment; /* increment the fill type counter */

                if (fill >= fill_size) /* fill type counter greater than the size of the fill_array or pattern_array? */
                {
                    fill = 0; /* at end of fill or patterns, start over */
                }

            }

        }

    }

    print_message (retval); /* print any error messages */
    exit (retval); /* exit with the error value */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}
